@@ -11,7 +11,6 @@ Inheritance diagram:
 # Distributed under the terms of the Modified BSD License.
 
 import abc
-import json
 import sys
 import traceback
 import warnings
@@ -30,6 +29,8 @@ from traitlets import (
     default, observe,
 )
 
+from typing import Any
+
 
 class DisplayFormatter(Configurable):
 
@@ -52,20 +53,23 @@ class DisplayFormatter(Configurable):
             else:
                 formatter.enabled = False
 
-    ipython_display_formatter = ForwardDeclaredInstance('FormatterABC')
-    @default('ipython_display_formatter')
+    ipython_display_formatter = ForwardDeclaredInstance("FormatterABC")  # type: ignore
+
+    @default("ipython_display_formatter")
     def _default_formatter(self):
         return IPythonDisplayFormatter(parent=self)
 
-    mimebundle_formatter = ForwardDeclaredInstance('FormatterABC')
-    @default('mimebundle_formatter')
+    mimebundle_formatter = ForwardDeclaredInstance("FormatterABC")  # type: ignore
+
+    @default("mimebundle_formatter")
     def _default_mime_formatter(self):
         return MimeBundleFormatter(parent=self)
 
     # A dict of formatter whose keys are format types (MIME types) and whose
     # values are subclasses of BaseFormatter.
     formatters = Dict()
-    @default('formatters')
+
+    @default("formatters")
     def _formatters_default(self):
         """Activate the default formatters."""
         formatter_classes = [
@@ -307,8 +311,8 @@ class BaseFormatter(Configurable):
     returned and this format type is not used.
     """
 
-    format_type = Unicode('text/plain')
-    _return_type = str
+    format_type = Unicode("text/plain")
+    _return_type: Any = str
 
     enabled = Bool(True).tag(config=True)
 
