@@ -1,4 +1,3 @@
-# encoding: utf-8
 """
 Utilities for working with strings and text.
 
@@ -13,16 +12,17 @@ import re
 import string
 import sys
 import textwrap
+import warnings
 from string import Formatter
 from pathlib import Path
 
-from IPython.utils import py3compat
+from typing import List, Dict, Tuple, Optional, cast, Sequence, Mapping, Any
 
-# datetime.strftime date format for ipython
-if sys.platform == 'win32':
-    date_format = "%B %d, %Y"
+if sys.version_info < (3, 12):
+    from typing_extensions import Self
 else:
-    date_format = "%B %-d, %Y"
+    from typing import Self
+
 
 class LSString(str):
     """String derivative with a special access attributes.
@@ -40,7 +40,11 @@ class LSString(str):
     Such strings are very useful to efficiently interact with the shell, which
     typically only understands whitespace-separated options for commands."""
 
-    def get_list(self):
+    __list: List[str]
+    __spstr: str
+    __paths: List[Path]
+
+    def get_list(self) -> List[str]:
         try:
             return self.__list
         except AttributeError:
@@ -49,7 +53,7 @@ class LSString(str):
 
     l = list = property(get_list)
 
-    def get_spstr(self):
+    def get_spstr(self) -> str:
         try:
             return self.__spstr
         except AttributeError:
@@ -58,12 +62,12 @@ class LSString(str):
 
     s = spstr = property(get_spstr)
 
-    def get_nlstr(self):
+    def get_nlstr(self) -> Self:
         return self
 
     n = nlstr = property(get_nlstr)
 
-    def get_paths(self):
+    def get_paths(self) -> List[Path]:
         try:
             return self.__paths
         except AttributeError:
@@ -78,8 +82,8 @@ class LSString(str):
 
 # def print_lsstring(arg):
 #     """ Prettier (non-repr-like) and more informative printer for LSString """
-#     print "LSString (.p, .n, .l, .s available). Value:"
-#     print arg
+#     print("LSString (.p, .n, .l, .s available). Value:")
+#     print(arg)
 #
 #
 # print_lsstring = result_display.register(LSString)(print_lsstring)
@@ -98,12 +102,16 @@ class SList(list):
     Any values which require transformations are computed only once and
     cached."""
 
-    def get_list(self):
+    __spstr: str
+    __nlstr: str
+    __paths: List[Path]
+
+    def get_list(self) -> Self:
         return self
 
     l = list = property(get_list)
 
-    def get_spstr(self):
+    def get_spstr(self) -> str:
         try:
             return self.__spstr
         except AttributeError:
@@ -112,7 +120,7 @@ class SList(list):
 
     s = spstr = property(get_spstr)
 
-    def get_nlstr(self):
+    def get_nlstr(self) -> str:
         try:
             return self.__nlstr
         except AttributeError:
@@ -121,7 +129,7 @@ class SList(list):
 
     n = nlstr = property(get_nlstr)
 
-    def get_paths(self):
+    def get_paths(self) -> List[Path]:
         try:
             return self.__paths
         except AttributeError:
@@ -237,7 +245,7 @@ class SList(list):
 
 # def print_slist(arg):
 #     """ Prettier (non-repr-like) and more informative printer for SList """
-#     print "SList (.p, .n, .l, .s, .grep(), .fields(), sort() available):"
+#     print("SList (.p, .n, .l, .s, .grep(), .fields(), sort() available):")
 #     if hasattr(arg,  'hideonce') and arg.hideonce:
 #         arg.hideonce = False
 #         return
@@ -254,7 +262,6 @@ def indent(instr,nspaces=4, ntabs=0, flatten=False):
 
     Parameters
     ----------
-
     instr : basestring
         The string to be indented.
     nspaces : int (default: 4)
@@ -268,7 +275,6 @@ def indent(instr,nspaces=4, ntabs=0, flatten=False):
 
     Returns
     -------
-
     str|unicode : string indented by ntabs and nspaces.
 
     """
@@ -339,7 +345,13 @@ ini_spaces_re = re.compile(r'^(\s+)')
 
 def num_ini_spaces(strng):
     """Return the number of initial spaces in a string"""
-
+    warnings.warn(
+        "`num_ini_spaces` is Pending Deprecation since IPython 8.17."
+        "It is considered fro removal in in future version. "
+        "Please open an issue if you believe it should be kept.",
+        stacklevel=2,
+        category=PendingDeprecationWarning,
+    )
     ini_spaces = ini_spaces_re.match(strng)
     if ini_spaces:
         return ini_spaces.end()
@@ -357,7 +369,7 @@ def format_screen(strng):
     return strng
 
 
-def dedent(text):
+def dedent(text: str) -> str:
     """Equivalent of textwrap.dedent that ignores unindented first line.
 
     This means it will still dedent strings like:
@@ -392,9 +404,15 @@ def wrap_paragraphs(text, ncols=80):
 
     Returns
     -------
-
     list of complete paragraphs, wrapped to fill `ncols` columns.
     """
+    warnings.warn(
+        "`wrap_paragraphs` is Pending Deprecation since IPython 8.17."
+        "It is considered fro removal in in future version. "
+        "Please open an issue if you believe it should be kept.",
+        stacklevel=2,
+        category=PendingDeprecationWarning,
+    )
     paragraph_re = re.compile(r'\n(\s*\n)+', re.MULTILINE)
     text = dedent(text).strip()
     paragraphs = paragraph_re.split(text)[::2] # every other entry is space
@@ -463,22 +481,30 @@ def strip_email_quotes(text):
 def strip_ansi(source):
     """
     Remove ansi escape codes from text.
-    
+
     Parameters
     ----------
     source : str
         Source to remove the ansi from
     """
+    warnings.warn(
+        "`strip_ansi` is Pending Deprecation since IPython 8.17."
+        "It is considered fro removal in in future version. "
+        "Please open an issue if you believe it should be kept.",
+        stacklevel=2,
+        category=PendingDeprecationWarning,
+    )
+
     return re.sub(r'\033\[(\d|;)+?m', '', source)
 
 
 class EvalFormatter(Formatter):
     """A String Formatter that allows evaluation of simple expressions.
-    
-    Note that this version interprets a : as specifying a format string (as per
+
+    Note that this version interprets a `:`  as specifying a format string (as per
     standard string formatting), so if slicing is required, you must explicitly
     create a slice.
-    
+
     This is to be used in templating cases, such as the parallel batch
     script templates, where simple arithmetic on arguments is useful.
 
@@ -526,11 +552,14 @@ class FullEvalFormatter(Formatter):
     """
     # copied from Formatter._vformat with minor changes to allow eval
     # and replace the format_spec code with slicing
-    def vformat(self, format_string:str, args, kwargs)->str:
+    def vformat(
+        self, format_string: str, args: Sequence[Any], kwargs: Mapping[str, Any]
+    ) -> str:
         result = []
-        for literal_text, field_name, format_spec, conversion in \
-                self.parse(format_string):
-
+        conversion: Optional[str]
+        for literal_text, field_name, format_spec, conversion in self.parse(
+            format_string
+        ):
             # output the literal text
             if literal_text:
                 result.append(literal_text)
@@ -546,10 +575,11 @@ class FullEvalFormatter(Formatter):
 
                 # eval the contents of the field for the object
                 # to be formatted
-                obj = eval(field_name, kwargs)
+                obj = eval(field_name, dict(kwargs))
 
                 # do any conversion on the resulting object
-                obj = self.convert_field(obj, conversion)
+                # type issue in typeshed, fined in https://github.com/python/typeshed/pull/11377
+                obj = self.convert_field(obj, conversion)  # type: ignore[arg-type]
 
                 # format the object and append to the result
                 result.append(self.format_field(obj, ''))
@@ -597,6 +627,9 @@ class DollarFormatter(FullEvalFormatter):
             # Re-yield the {foo} style pattern
             yield (txt + literal_txt[continue_from:], field_name, format_spec, conversion)
 
+    def __repr__(self):
+        return "<DollarFormatter>"
+
 #-----------------------------------------------------------------------------
 # Utils to columnize a list of string
 #-----------------------------------------------------------------------------
@@ -612,7 +645,9 @@ def _col_chunks(l, max_rows, row_first=False):
             yield l[i:(i + max_rows)]
 
 
-def _find_optimal(rlist, row_first=False, separator_size=2, displaywidth=80):
+def _find_optimal(
+    rlist: List[str], row_first: bool, separator_size: int, displaywidth: int
+) -> Dict[str, Any]:
     """Calculate optimal info to columnize a list of string"""
     for max_rows in range(1, len(rlist) + 1):
         col_widths = list(map(max, _col_chunks(rlist, max_rows, row_first)))
@@ -635,12 +670,18 @@ def _get_or_default(mylist, i, default=None):
         return mylist[i]
 
 
-def compute_item_matrix(items, row_first=False, empty=None, *args, **kwargs) :
+def compute_item_matrix(
+    items: List[str],
+    row_first: bool = False,
+    empty: Optional[str] = None,
+    *,
+    separator_size: int = 2,
+    displaywidth: int = 80,
+) -> Tuple[List[List[int]], Dict[str, int]]:
     """Returns a nested list, and info to columnize items
 
     Parameters
     ----------
-
     items
         list of strings to columize
     row_first : (default False)
@@ -655,14 +696,11 @@ def compute_item_matrix(items, row_first=False, empty=None, *args, **kwargs) :
 
     Returns
     -------
-
     strings_matrix
-
         nested list of string, the outer most list contains as many list as
         rows, the innermost lists have each as many element as columns. If the
         total number of elements in `items` does not equal the product of
         rows*columns, the last element of some lists are filled with `None`.
-
     dict_info
         some info to make columnize easier:
 
@@ -687,29 +725,44 @@ def compute_item_matrix(items, row_first=False, empty=None, *args, **kwargs) :
         In [5]: all((info[k] == ideal[k] for k in ideal.keys()))
         Out[5]: True
     """
-    info = _find_optimal(list(map(len, items)), row_first, *args, **kwargs)
-    nrow, ncol = info['max_rows'], info['num_columns']
+    warnings.warn(
+        "`compute_item_matrix` is Pending Deprecation since IPython 8.17."
+        "It is considered fro removal in in future version. "
+        "Please open an issue if you believe it should be kept.",
+        stacklevel=2,
+        category=PendingDeprecationWarning,
+    )
+    info = _find_optimal(
+        list(map(len, items)),  # type: ignore[arg-type]
+        row_first,
+        separator_size=separator_size,
+        displaywidth=displaywidth,
+    )
+    nrow, ncol = info["max_rows"], info["num_columns"]
     if row_first:
         return ([[_get_or_default(items, r * ncol + c, default=empty) for c in range(ncol)] for r in range(nrow)], info)
     else:
         return ([[_get_or_default(items, c * nrow + r, default=empty) for c in range(ncol)] for r in range(nrow)], info)
 
 
-def columnize(items, row_first=False, separator='  ', displaywidth=80, spread=False):
-    """ Transform a list of strings into a single string with columns.
+def columnize(
+    items: List[str],
+    row_first: bool = False,
+    separator: str = "  ",
+    displaywidth: int = 80,
+    spread: bool = False,
+) -> str:
+    """Transform a list of strings into a single string with columns.
 
     Parameters
     ----------
     items : sequence of strings
         The strings to process.
-
     row_first : (default False)
         Whether to compute columns for a row-first matrix instead of
         column-first (default).
-
     separator : str, optional [default is two spaces]
         The string that separates columns.
-
     displaywidth : int, optional [default is 80]
         Width of the display in number of characters.
 
@@ -717,14 +770,29 @@ def columnize(items, row_first=False, separator='  ', displaywidth=80, spread=Fa
     -------
     The formatted string.
     """
+    warnings.warn(
+        "`columnize` is Pending Deprecation since IPython 8.17."
+        "It is considered for removal in future versions. "
+        "Please open an issue if you believe it should be kept.",
+        stacklevel=2,
+        category=PendingDeprecationWarning,
+    )
     if not items:
-        return '\n'
-    matrix, info = compute_item_matrix(items, row_first=row_first, separator_size=len(separator), displaywidth=displaywidth)
+        return "\n"
+    matrix: List[List[int]]
+    matrix, info = compute_item_matrix(
+        items,
+        row_first=row_first,
+        separator_size=len(separator),
+        displaywidth=displaywidth,
+    )
     if spread:
-        separator = separator.ljust(int(info['optimal_separator_width']))
-    fmatrix = [filter(None, x) for x in matrix]
-    sjoin = lambda x : separator.join([ y.ljust(w, ' ') for y, w in zip(x, info['column_widths'])])
-    return '\n'.join(map(sjoin, fmatrix))+'\n'
+        separator = separator.ljust(int(info["optimal_separator_width"]))
+    fmatrix: List[filter[int]] = [filter(None, x) for x in matrix]
+    sjoin = lambda x: separator.join(
+        [y.ljust(w, " ") for y, w in zip(x, cast(List[int], info["column_widths"]))]
+    )
+    return "\n".join(map(sjoin, fmatrix)) + "\n"
 
 
 def get_text_list(list_, last_sep=' and ', sep=", ", wrap_item_with=""):
